@@ -86,24 +86,15 @@ Usage is
 python3 -m skipole mynewproj /path/to/projectfiles
 
 Which creates a directory /path/to/projectfiles
-containing three sub directories:
-
-mynewproj - a subdirectory containing a new project
-skis - a project serving needed javascript files, necessary for all projects
-skiadmin - a project used to help develop 'mynewproj'
+containing sub directory mynewproj - containing project data, and file
+mynewproj.py where your code will be developed.
 
 You should replace 'mynewproj' with your preferred name for a new project.
-This is an optional argument. If not given then only the skis and skiadmin
-projects will be created.
 
-The path "/path/to/projectfiles" must be given, and is the path to a directory
-where you will develop your project. Multiple projects can be created in one
-'projectfiles' directory, or you could have multiple such directories holding
-different projects.
-
-If the directory already exists, any existing skiadmin and skis projects will
-be overwritten, this facility is used to upgrade skis and skiadmin if a new
-version of skipole is installed.
+The path "/path/to/projectfiles" is the path to a directory where you will
+develop your project. Multiple projects can be created in one 'projectfiles'
+directory, or you could have multiple such directories holding different
+projects.
 
 If mynewproj already exists in the directory, it will not be changed.
 
@@ -206,10 +197,10 @@ url - path where this project will be served, typically '/'
         "The instance is callable"
         return self._skipoleproject(environ, start_response)
 
-    def add_project(self, proj, url=None):
+    def add_project(self, proj, url=None, check_cookies=None):
         """Add a sub project to this root project, returns the sub project url
            proj is the sub project WSGIApplication object."""
-        return self._skipoleproject.add_project(proj._skipoleproject, url)
+        return self._skipoleproject.add_project(proj._skipoleproject, url, check_cookies)
 
     def set_accesstextblocks(self, accesstextblocks):
         """Set an instance of a class which reads and writes TextBlocks. The default class is defined in the skipole.textblocks module,
@@ -252,20 +243,13 @@ def use_submit_list(submit_data):
             sdpackage = sdmodule.__name__
         try:
             submitmodule = import_module(submitpath, sdpackage)
-        except Exception:
-            if skiboot.get_debug():
-                exc_type, exc_value, exc_traceback = sys.exc_info()
-                str_list = traceback.format_exception(exc_type, exc_value, exc_traceback)
-                message = ''
-                for item in str_list:
-                    message += item
-                raise ServerError(message)
-            raise ServerError("Unable to import project module defined as %s in submit list" % (submitpath,))
+        except Exception as e:
+            raise ServerError("Unable to import project module defined as %s in submit list" % (submitpath,)) from e
         # now obtain and run the specified function
         try:
             submitfunc = getattr(submitmodule, skicall.submit_list[-1])
-        except Exception:
-            raise ServerError("submit_list package %s found, but the required function %s is not recognised" % (submitpath, skicall.submit_list[-1]))
+        except Exception as e:
+            raise ServerError("submit_list package %s found, but the required function %s is not recognised" % (submitpath, skicall.submit_list[-1])) from e
         return submitfunc(skicall)
     return submit_function
 
