@@ -65,7 +65,7 @@ class ParentPage(object):
             return
         if self.ident_data:
             # encode the ident data as base64, and append it as a string to the page ident
-            b64binarydata = urlsafe_b64encode(self.ident_data.encode('ascii')).rstrip(b"=")  # removes final '=' padding
+            b64binarydata = urlsafe_b64encode(self.ident_data.encode('utf-8')).rstrip(b"=")  # removes final '=' padding
             return str(self.ident) + '_' + b64binarydata.decode('ascii')
         else:
             return str(self.ident)        
@@ -1210,6 +1210,8 @@ class FilePage(ParentPage):
         t, e = mimetypes.guess_type(name, strict=False)
         if t:
             return t
+        else:
+            return "application/octet-stream"
 
     def set_mimetype(self, mt):
         self._mimetype = mt
@@ -1584,7 +1586,7 @@ class JSON(ParentPage):
             return [json.dumps(self.jsondict).encode('UTF-8')]
         if self.ident_data:
             # encode the ident data as base64
-            b64binarydata = urlsafe_b64encode(self.ident_data.encode('ascii')).rstrip(b"=")  # removes final '=' padding
+            b64binarydata = urlsafe_b64encode(self.ident_data.encode('utf-8')).rstrip(b"=")  # removes final '=' padding
             self.content['ident_data'] = b64binarydata.decode('ascii')
         elif 'ident_data' in self.content:
             del self.content['ident_data']
@@ -1596,7 +1598,7 @@ class JSON(ParentPage):
             return json.dumps(self.jsondict)
         if self.ident_data:
             # encode the ident data as base64
-            b64binarydata = urlsafe_b64encode(self.ident_data.encode('ascii')).rstrip(b"=")  # removes final '=' padding
+            b64binarydata = urlsafe_b64encode(self.ident_data.encode('utf-8')).rstrip(b"=")  # removes final '=' padding
             self.content['ident_data'] = b64binarydata.decode('ascii')
         elif 'ident_data' in self.content:
             del self.content['ident_data']
