@@ -11,7 +11,7 @@ call and is passed as an argument to the user functions
 """
 
 
-import copy, os, cgi, collections, html, pprint, json, shutil, uuid, sys, traceback, re
+import copy, os, cgi, collections, html, pprint, json, shutil, uuid, sys, traceback, re, pathlib, mimetypes
 
 from base64 import urlsafe_b64decode
 
@@ -80,6 +80,10 @@ class SkipoleProject(object):
         # initially, assume this is the root projcet, and sub projects can be added
         self.rootproject = True
 
+        # A check cookies function can be set in this project if it is added as a sub-project
+        # initially it is None
+        self.check_cookies = None
+
         # initial values, will be set from the json file
         self.brief = "Project %s" % project
         self.version = "0.0.0"
@@ -195,7 +199,7 @@ class SkipoleProject(object):
         s_h_data = None
         try:
             if 'PATH_INFO' in environ:
-                path = environ['PATH_INFO'].lower()
+                path = environ['PATH_INFO']
             else:
                 raise ServerError(message="Invalid path", code=9034)
 
@@ -209,8 +213,34 @@ class SkipoleProject(object):
                 if (path.find(projurl) == 0) or (path + "/" == projurl):
                     # this url is within a sub project
                     subproj = self.subprojects[proj]
-                    s_h_data = subproj.proj_respond(environ, projurl, path, lang, received_cookies)
-                    break
+                    if subproj.check_cookies is None:
+                        # there is no check_cookies function, so no divertedcall. Call proj_respond of the sub project
+                        s_h_data = subproj.proj_respond(environ, projurl, path, lang, received_cookies)
+                        break
+                    # the subproj has a check_cookies function, call it. Call proj_respond of the sub project
+                    divertedcall = subproj.check_cookies(received_cookies, self.proj_data)
+                    if divertedcall is None:
+                        # check_cookies returns None, so no diversion
+                        s_h_data = subproj.proj_respond(environ, projurl, path, lang, received_cookies)
+                        break
+                    # a divertedcall has been returned, it can be integer/tuple/label. Convert to ident
+                    divertedcall = skiboot.find_ident(divertedcall, proj_ident=self._proj_ident)
+                    if divertedcall is None:
+                        # no ident found, so this is a ulr_not_found, leave s_h_data as None
+                        break
+                    # a divertedcall ident is given, but it could be to a page in this root project or any sub project
+                    if divertedcall[0] == self._proj_ident:
+                        # the diversion is to an ident of this root project
+                        s_h_data = self.proj_respond(environ, self.url, path, lang, received_cookies, divertedcall)
+                        break
+                    elif divertedcall[0] in self.subprojects:
+                        # the diversion is to an ident of a sub project, identify the sub project
+                        subproj = self.subprojects[divertedcall[0]]
+                        s_h_data = subproj.proj_respond(environ, self._subproject_paths[divertedcall[0]], path, lang, received_cookies, divertedcall)
+                        break
+                    else:
+                        # should never occur, but if it does, leave s_h_data as None
+                        break
             else:
                 # the call is for a page in this root project
                 s_h_data = self.proj_respond(environ, self.url, path, lang, received_cookies)
@@ -224,11 +254,18 @@ class SkipoleProject(object):
                 return self._url_not_found(environ, path, lang)
 
         except ServerError as e:
+            # if debug is enabled, expand the exception message to include the exception trace
+            if skiboot.get_debug():
+                e.message += "\n"
+                exc_type, exc_value, exc_traceback = sys.exc_info()
+                str_list = traceback.format_exception(exc_type, exc_value, exc_traceback)
+                for item in str_list:
+                    e.message += item
             # ServerError has occurred, return the server error page
             page = self._system_page("server_error")
             if (not page) or (page.page_type != "TemplatePage"):
                 # return the default server error page
-                return self.default_server_error_page(e)
+                return self.default_server_error_page(e.message, e.code)
             # import any sections
             page.import_sections()
             # show message passed by the exception
@@ -252,7 +289,7 @@ class SkipoleProject(object):
             status, headers = page.get_status()
             data = page.data()
             if not data:
-                return self.default_server_error_page(e)
+                return self.default_server_error_page(e.message, e.code)
             # return page data
             s_h_data = e.status, headers, data
 
@@ -260,7 +297,7 @@ class SkipoleProject(object):
 
 
 
-    def proj_respond(self, environ, projurl, path, lang, received_cookies):
+    def proj_respond(self, environ, projurl, path, lang, received_cookies, divertedcall=None):
         """Gets any received form data, and parses the ident field if present to find the caller page and ident_data
            Calls start call, and depending on the returned page, calls the project status_headers_data method"""
 
@@ -302,18 +339,18 @@ class SkipoleProject(object):
                             b64binarydata = ident_parts[2].encode('ascii') # get the submitted data and convert to binary
                             # add padding
                             b64binarydata = b64binarydata + b"=" * (4-len(b64binarydata)%4)
-                            ident_data = urlsafe_b64decode(b64binarydata).decode('ascii') # b64 decode, and convert to string
+                            ident_data = urlsafe_b64decode(b64binarydata).decode('utf-8') # b64 decode, and convert to string
                     except Exception:
                         caller_page = None
                     if caller_page is None:
-                       raise ValidateError(message="Form data not accepted, (received ident is not valid)")
+                        raise ValidateError(message="Form data not accepted, (received ident is not valid)")
                     if caller_page.page_type != 'TemplatePage':
                         raise ValidateError(message="Form data not accepted, (caller page ident is not a template page)")
 
         except ValidateError as e:
             page = self._system_page("validate_error")
             if (not page) or (page.page_type != "TemplatePage"):
-                return self.default_validate_error_page(e)
+                return self.default_validate_error_page(e.message)
             # import any sections
             page.import_sections()
             # show message passed by the exception
@@ -323,7 +360,7 @@ class SkipoleProject(object):
             status, headers = page.get_status()
             data = page.data()
             if not data:
-                return self.default_validate_error_page(e)
+                return self.default_validate_error_page(e.message)
             # return page data
             return e.status, headers, data
 
@@ -331,7 +368,10 @@ class SkipoleProject(object):
         # so caller_page could be either given, or could be None
 
         # get the called ident, could be None
-        ident = self.page_ident_from_path(projurl, path)
+        if divertedcall is None:
+            ident = self.page_ident_from_path(projurl, path)
+        else:
+            ident = divertedcall
 
         if caller_page:
             caller_page_ident = caller_page.ident
@@ -355,6 +395,37 @@ class SkipoleProject(object):
             # URL NOT FOUND and start_call does not divert
             return None
 
+        if isinstance(pident, pathlib.Path):
+            # pident is a path to a file on the server, so serve that file by returning status, headers, data
+            try:
+                if ('status' in skicall.page_data) and skicall.page_data['status']:
+                    status = skicall.page_data['status']
+                else:
+                    status = "200 OK"
+                if ('headers' in skicall.page_data) and skicall.page_data['headers']:
+                    headers = skicall.page_data['headers']
+                else:
+                    headers = [('content-length', str(pident.stat().st_size))]
+                    if 'mimetype' in skicall.page_data:
+                        headers.append(('content-type', skicall.page_data['mimetype']))
+                    else:
+                        t, e = mimetypes.guess_type(pident.name, strict=False)
+                        if t:
+                            headers.append(('content-type', t))
+                        else:
+                            headers.append(('content-type', "application/octet-stream"))
+                    if 'enable_cache' in skicall.page_data:
+                        if skicall.page_data['enable_cache']:
+                            headers.append(('cache-control', 'max-age=3600'))
+                        else:
+                            headers.append(('cache-control','no-cache, no-store, must-revalidate'))
+                            headers.append(('Pragma', 'no-cache'))
+                            headers.append(('Expires', '0'))
+                data = _read_server_file(environ, pident)
+            except Exception as e:
+                raise ServerError(message=f"Failed to read file {pident}", code=9035) from e
+            return status, headers, data
+
         # pident is the ident of the diverted page or a label or url string
 
         # get the page from pident
@@ -418,7 +489,7 @@ class SkipoleProject(object):
         except ValidateError as e:
             page = self._system_page("validate_error")
             if (not page) or (page.page_type != "TemplatePage"):
-                return self.default_validate_error_page(e)
+                return self.default_validate_error_page(e.message)
             # import any sections
             page.import_sections()
             # show message passed by the exception
@@ -428,7 +499,7 @@ class SkipoleProject(object):
             status, headers = page.get_status()
             data = page.data()
             if not data:
-                return self.default_validate_error_page(e)
+                return self.default_validate_error_page(e.message)
             # return page data
             return e.status, headers, page.data()
 
@@ -465,6 +536,10 @@ class SkipoleProject(object):
             # being set by the users own start_call function
             new_called_ident = self.start_call(called_ident, skicall)
 
+            # if new_called_ident is a Path object, then check if a file exists
+            if isinstance(new_called_ident, pathlib.Path):
+                if not new_called_ident.is_file():
+                    new_called_ident = None
             # convert returned tuple to an Ident object
             if isinstance(new_called_ident, int):
                 new_called_ident = (self._proj_ident, new_called_ident)
@@ -473,15 +548,8 @@ class SkipoleProject(object):
             # could be a label
         except ServerError as e:
             raise e
-        except Exception:
-            message = "Invalid exception in start_call function."
-            if skiboot.get_debug():
-                message += "\n"
-                exc_type, exc_value, exc_traceback = sys.exc_info()
-                str_list = traceback.format_exception(exc_type, exc_value, exc_traceback)
-                for item in str_list:
-                    message += item
-            raise ServerError(message, code=9040)
+        except Exception as e:
+            raise ServerError(message = "Invalid exception in start_call function.", code=9040) from e
         return new_called_ident, skicall
 
 
@@ -625,16 +693,10 @@ class SkipoleProject(object):
                 if skicall._lang_cookie:
                     page.language_cookie = skicall._lang_cookie
         except GoTo as e:
-            raise ServerError("Invalid GoTo exception in end_call", code=9049)
-        except Exception:
+            raise ServerError("Invalid GoTo exception in end_call", code=9049) from e
+        except Exception as e:
             message = "Invalid exception in end_call function."
-            if skiboot.get_debug():
-                message += "\n"
-                exc_type, exc_value, exc_traceback = sys.exc_info()
-                str_list = traceback.format_exception(exc_type, exc_value, exc_traceback)
-                for item in str_list:
-                    message += item
-            raise ServerError(message, code=9050)
+            raise ServerError(message, code=9050) from e
 
         # import any sections
         page.import_sections(skicall.page_data)
@@ -1164,9 +1226,6 @@ class SkipoleProject(object):
         return status, headers, page.data()
 
 
-
-
-
     def _url_not_found(self, environ, path, lang):
         "Used to return the url not found page"
         page = self._system_page("url_not_found")
@@ -1190,28 +1249,25 @@ class SkipoleProject(object):
         return '404 Not Found', [('content-type', 'text/html')], [page_content]
 
 
-    def default_server_error_page(self, e):
+    def default_server_error_page(self, message='', code=0):
         "Given a ServerError exception, return a default status,headers,data"
-        text_start = "<!DOCTYPE HTML>\n<html>\n<p>SERVER ERROR</p>\n<p>Error code : %s</p>\n" % (e.code,)
-        if e.message:
-            page_text = text_start + "<p>%s</p>\n</html>" % (html.escape(e.message),)
+        text_start = "<!DOCTYPE HTML>\n<html>\n<p>SERVER ERROR</p>\n<p>Error code : %s</p>\n" % (code,)
+        if message:
+            page_text = text_start + "<pre>%s</pre>\n</html>" % (html.escape(message),)
         else:
             page_text = text_start + "</html>"
         return '500 Internal Server Error', [('content-type', 'text/html')], [page_text.encode('ascii', 'xmlcharrefreplace')]
 
 
-    def default_validate_error_page(self, e):
+    def default_validate_error_page(self, message):
         "Given a ValidateError exception, return a default status,headers,data"
-        if e.message:
-            page_text = "<!DOCTYPE HTML>\n<html>\n<p>VALIDATION ERROR</p>\n<p>%s</p>\n</html>" % (html.escape(e.message),)
+        if message:
+            page_text = "<!DOCTYPE HTML>\n<html>\n<p>VALIDATION ERROR</p>\n<p>%s</p>\n</html>" % (html.escape(message),)
         else:
             page_text = "<!DOCTYPE HTML>\n<html>\n<p>VALIDATION ERROR</p>\n</html>"
         return '400 Bad Request', [('content-type', 'text/html')], [page_text.encode('ascii', 'xmlcharrefreplace')]
 
 
-
-
-
     def page_ident_from_path(self, projurl, path):
         """Tests if ident exists in the cache, return it, if not, call self.root.page_ident_from_path
            and cach the result, then return the ident. If no ident found, or if ident within a restricted folder, return None."""
@@ -1263,12 +1319,16 @@ class SkipoleProject(object):
         "Returns a list of subproject idents"
         return [i for i in self.subproject_paths]
 
-    def add_project(self, proj, url=None):
+    def add_project(self, proj, url=None, check_cookies=None):
         """Add a project to self, returns the url
            proj is the sub project application.
            This adds a reference to the project to the subproject_paths, returns the sub project path"""
         if not self.rootproject:
            raise ValidateError(message="Cannot add to a sub project")
+
+        if check_cookies is not None:
+            if not callable(check_cookies):
+                raise ValidateError(message="If given check_cookies must be callable as check_cookies(received_cookies, proj_data)")
         proj_id = proj.proj_ident
 
         # get a copy of the {proj_id:url} subproject_paths dictionary, and this projects url
@@ -1301,6 +1361,8 @@ class SkipoleProject(object):
         proj._subproject_paths = collections.OrderedDict()
         proj.subprojects = {}
         self.subprojects[proj_id] = proj
+        # set check_cookies function into the sub project - perhaps could 
+        proj.check_cookies = check_cookies
         return url
 
     @property
@@ -1400,3 +1462,61 @@ class SkiCall(object):
         return {proj_ident:proj.url for proj_ident, proj in all_projects.items()}
 
 
+    def map_url_to_server(self, urlfolder, serverfolder, mimetype=""):
+        """Generally called in the start_call function. Maps a url folder such as
+           "/special/css" to a server folder such as "/home/user/thisproject/css"
+           if a call is made to, say "/special/css/myfile.css" this function will return a 
+           pathlib.Path object to the file "/home/user/thisproject/css/myfile.css" if such a file
+           exists. If not then None is returned.
+           If start_call returns such a pathlib.Path, then the framework will serve the file.
+           A mimetype can be given, otherwise it will be guessed.
+           An example of usage is:
+            def start_call(called_ident, skicall):
+                servedfile = skicall.map_url_to_server("/special/css", "/home/user/thisproject/css")
+                if servedfile:
+                    return servedfile
+                return called_ident
+        """
+        server_folder = pathlib.Path(serverfolder).expanduser().resolve()
+        if not server_folder.is_dir():
+            return
+        if self.path.startswith(urlfolder):
+            url_folder = pathlib.Path(urlfolder)
+            # the path requested
+            path = pathlib.Path(self.path)
+            # path must have more elements than url_folder, as it should have at least the filename
+            if len(path.parts) <= len(url_folder.parts):
+                return
+            servedfileparts = server_folder.parts + path.parts[len(url_folder.parts):]
+            servedfile = pathlib.Path(*servedfileparts)
+            if not servedfile.is_file():
+                return
+            if mimetype:
+                self.page_data['mimetype'] = mimetype
+            else:
+                mimetypes.init()
+                t, e = mimetypes.guess_type(path.name, strict=False)
+                if t:
+                    self.page_data['mimetype'] = t
+                else:
+                    self.page_data['mimetype'] = "application/octet-stream"
+            return servedfile
+        return
+
+def _readfile(filepath, size):
+    "Return a generator reading the file"
+    with filepath.open("rb") as f:
+        data = f.read(size)
+        while data:
+            yield data
+            data = f.read(size)
+
+def _read_server_file(environ, filepath, size=32768):
+    "Returns an iterator of the file"   
+    if 'wsgi.file_wrapper' in environ:
+        f = filepath.open("rb")
+        return environ['wsgi.file_wrapper'](f, size)
+    else:
+        return _readfile(filepath, size)
+
+
