@@ -4,7 +4,7 @@
       further input fields.  The module also has an Hidden Field and Submit Button widgets, which can be inserted into
      a form. """
 
-from .. import skiboot, tag, excepts
+from .. import skiboot, tag
 from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
@@ -249,6 +249,8 @@ class Form1(Widget):
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a submit event handler"""
+        # this ensures any input text widgets added to the container, get local validation
+        # when the form is submitted
         jscript = """  $('#{ident} form').on("submit", function(e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
